#include "lib/sc12/rtos.h"


/** Queue length for messages */
#define  MESSAGE_QUEUE_SIZE   16

class CommandMessages
{
	public:
      int Count();
		int Get();
      void Put(int iMessageID);
   private:
      /** queue for messages */
      int  queue[MESSAGE_QUEUE_SIZE];
      int       		qHead, qCount;
};



/**
 * Check for currently available keycodes. Any available
 * keycodes can be retrieved with GetKey()
 * \sa      GetKey()
 * \return  number of available keycodes.
 */
int CommandMessages::Count()
{
  #ifdef RC_DEBUG
    ddump();
  #endif
  return (int)qCount;
}


/**
 * Get a keycode from the queue. This function will block until
 * a keycode is received. Use the MessageCount() function to check
 * if there are codes in the queue.
 * \sa      MessageCount()
 * \return  valid keycode != 0
 */

 int CommandMessages::Get()
{
  while (!Count()) {
    RTX_Sleep_Time(10);
  }

  int key;
  key    = queue[qHead++];
  qHead %= MESSAGE_QUEUE_SIZE;
  qCount--;

  return key;
}

/**
 * Put a keycode to the queue.
 */
void CommandMessages::Put(int key)
{
	if (key && qCount < MESSAGE_QUEUE_SIZE)
   {
      queue[(qHead+qCount) % MESSAGE_QUEUE_SIZE] = key;
      qCount++;
	}
}
